package com.simuwang.manage.task;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.EasyExcel;
import com.simuwang.base.common.conts.DateConst;
import com.simuwang.base.config.CompetitionConfig;
import com.simuwang.base.pojo.dto.FundNavDataDTO;
import com.simuwang.manage.service.FundNavService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class GenerateNavDataTask {

    private static final Logger log = LoggerFactory.getLogger(GenerateNavDataTask.class);

    private final FundNavService fundNavService;
    private final CompetitionConfig competitionConfig;

    public GenerateNavDataTask(FundNavService fundNavService, CompetitionConfig competitionConfig) {
        this.fundNavService = fundNavService;
        this.competitionConfig = competitionConfig;
    }

     @Scheduled(cron = "0 0/1 * * * ?")
    public void runTask() {
        List<FundNavDataDTO> navDataDTOList = fundNavService.getFungNavData();
        if (CollUtil.isEmpty(navDataDTOList)) {
            return;
        }
        String fileName = competitionConfig.getDirectory() + "基金净值采集结果明细_" + DateUtil.format(new Date(), DateConst.YYYYMMDDHHMMSS24) + ".xlsx";
        EasyExcel.write(fileName, FundNavDataDTO.class)
                .sheet("基金净值采集结果明细")
                .doWrite(navDataDTOList);
        log.info("基金净值采集结果明细表格已输出... -> 文件:{}", fileName);

        Integer method = competitionConfig.getMethod();
        // 通过邮件发送"基金净值采集结果明细表格"
        if(method == 1){

        }
    }
}
